use anyhow::Result;
use async_graphql::SimpleObject;
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, PgPool};

#[derive(SimpleObject, Debug, FromRow, Deserialize, Serialize, sqlx::Type)]
pub struct Hersteller {
    /// Die Datenbank-ID
    pub hersteller_id: i32,

    /// Der Name eines Herstellers
    name: String,
}

impl Hersteller {
    pub async fn create(pool: &PgPool, name: &str) -> Result<Hersteller> {
        let row = sqlx::query!(
            "INSERT INTO hersteller(name) VALUES ($1) RETURNING hersteller_id",
            name
        )
        .fetch_one(pool)
        .await?;

        Ok(Hersteller {
            hersteller_id: row.hersteller_id,
            name: name.to_string(),
        })
    }

    pub async fn read_one(pool: &PgPool, id: &i32) -> Result<Hersteller> {
        let row = sqlx::query_as!(
            Hersteller,
            "SELECT * FROM hersteller WHERE hersteller_id = $1",
            id
        )
        .fetch_one(pool)
        .await?;

        Ok(row)
    }

    pub async fn read_all(pool: &PgPool) -> Result<Vec<Hersteller>> {
        let rows = sqlx::query_as!(Hersteller, "SELECT hersteller_id, name FROM hersteller")
            .fetch_all(pool)
            .await?;

        Ok(rows)
    }

    pub async fn update(pool: &PgPool, id: &i32, name: &str) -> Result<Hersteller> {
        sqlx::query!(
            "UPDATE hersteller SET name=$1 WHERE hersteller_id = $2",
            name,
            id
        )
        .execute(pool)
        .await?;

        let t = Hersteller::read_one(pool, id).await?;

        Ok(t)
    }

    pub async fn delete(pool: &PgPool, id: &i32) -> Result<()> {
        sqlx::query!("DELETE FROM hersteller WHERE hersteller_id = $1", id)
            .execute(pool)
            .await?;

        Ok(())
    }
}
